@@ -8,6 +8,7 @@
 #include <unistd.h>
 #include <sys/mman.h>
 #include <sys/random.h>
+#include <linux/byteorder/little_endian.h>
 
 #include "gpt.h"
 #include "ptypes.h"
@@ -34,10 +35,18 @@ static const unsigned char GPT_PROTECTIVE_MBR[LBA_SIZE - MBR_OFFSET] =
 static const unsigned char gpt_signature[8] =
  "\x45\x46\x49\x20\x50\x41\x52\x54";
 
+uint32_t host_to_le32(uint32_t x){
+  return __cpu_to_le32(x);
+}
+
+uint64_t host_to_le64(uint64_t x){
+  return __cpu_to_le64(x);
+}
+
 #define MINIMUM_GPT_ENTRIES 128
 
 int update_crc(gpt_header *head, const gpt_entry *gpes){
-  size_t hs = head->headsize; // FIXME little-endian; swap on BE machines
+  size_t hs = host_to_le32(head->headsize);
   // partition entry size must be a positive multiple of 128 (usually 128)
   if(head->partsize == 0 || head->partsize % MINIMUM_GPT_ENTRIES){
     return -1;
@@ -45,9 +54,9 @@ int update_crc(gpt_header *head, const gpt_entry *gpes){
   if(head->partcount < MINIMUM_GPT_ENTRIES){
     return -1;
   }
-  head->partcrc = crc32(0, (const void*)gpes, head->partcount * head->partsize);
+  head->partcrc = host_to_le32(crc32(0, (const void*)gpes, head->partcount * head->partsize));
   head->crc = 0;
-  head->crc = crc32(0, (void*)head, hs);
+  head->crc = host_to_le32(crc32(0, (const void*)head, hs));
   return 0;
 }
 
@@ -66,7 +75,7 @@ update_backup(int fd, const gpt_header *ghead, unsigned gptlbas, uint64_t lbas,
   mapsize = lbasize * gptlbas + mapoff;
   mapsize = pgsize * (mapsize / pgsize + !!mapoff);
   if((map = mmap(NULL, mapsize, PROT_READ|PROT_WRITE, MAP_SHARED, fd,
-        absdevoff - mapoff)) == MAP_FAILED){
+                 absdevoff - mapoff)) == MAP_FAILED){
     diag("Error mapping %zub at %d (%s?)\n", mapsize, fd, strerror(errno));
     return -1;
   }
@@ -75,8 +84,7 @@ update_backup(int fd, const gpt_header *ghead, unsigned gptlbas, uint64_t lbas,
     // Copy the partition table entries -- all but the first of the
     // primary header's sectors to all but the last of the backup
     // header's sectors.
-    memcpy((char *)map + mapoff, (char *)ghead + lbasize,
-      (gptlbas - 1) * lbasize);
+    memcpy((char *)map + mapoff, (char *)ghead + lbasize, (gptlbas - 1) * lbasize);
     // Copy the header, always a single LBA sector
     gh = (gpt_header *)((char *)map + lbasize * (gptlbas - 1) + mapoff);
     memcpy(gh, ghead, lbasize);
@@ -109,14 +117,15 @@ int initialize_gpt(gpt_header *gh, size_t lbasize, uint64_t backuplba,
     diag("Illegal backup LBA %ju\n", (uintmax_t)backuplba);
     return -1;
   }
-  if(lbasize == 0){
+  if(lbasize == 0 || lbasize < sizeof(*gh)){
     diag("Illegal LBA size %zu\n", lbasize);
     return -1;
   }
+  memset(gh, 0, lbasize);
   memcpy(&gh->signature, gpt_signature, sizeof(gh->signature));
-  gh->revision = 0x10000u;
+  gh->revision = host_to_le32(0x00010000ul);
   // FIXME ought be the lbasize and greater than 92, but everyone uses 92 :/
-  gh->headsize = sizeof(*gh);
+  gh->headsize = host_to_le32(sizeof(*gh));
   gh->reserved = 0;
   gh->lba = 1;
   // ->crc is set by update_crc()
@@ -135,9 +144,6 @@ int initialize_gpt(gpt_header *gh, size_t lbasize, uint64_t backuplba,
   gh->partcount = MINIMUM_GPT_ENTRIES;
   gh->partsize = sizeof(gpt_entry);
   // ->partcrc is set by update_crc()
-  if(lbasize > sizeof(*gh)){
-    memset((char *)gh + sizeof(*gh), 0, lbasize - sizeof(*gh));
-  }
   return 0;
 }
 
